import { unsafeHTML } from 'lit/directives/unsafe-html.js';

import markdownit from 'markdown-it';
import mila from 'markdown-it-link-attributes';
import type { PostProcessorRegistry } from '../../libs/PostProcessorRegistry.ts';

const MarkdownMixin = {
  name: 'markdown-mixin',
  created() {
    this.listValueTransformations.attach(
      this.transformValue.bind(this),
      'MarkdownMixin:transformValue',
    );
  },
  transformValue(
    value: string,
    listValueTransformations: PostProcessorRegistry,
  ) {
    let newValue: any = '';
    if (value) {
      const md = markdownit({
        breaks: true,
        html: false,
        linkify: true,
      });

      md.use(mila, {
        attrs: {
          target: '_blank',
          rel: 'noopener',
        },
      });

      const cleanedText = value.replace(/\*\*\s*(.*?)\s*\*\*/g, '**$1**');
      const html = md.render(cleanedText);
      newValue = unsafeHTML(html);
    }

    const nextProcessor = listValueTransformations.shift();
    if (nextProcessor) nextProcessor(newValue, listValueTransformations);
  },
};

export { MarkdownMixin };
